#!/usr/bin/env python

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import curses

import events
import filelist
import window

from helper import debug

class filelistwin(window.window):
    def __init__(self,
                 screen, h, w, y, x, hasscrollbar,
                 mp3basedir, songdb, keybindings):
        self.mp3base = mp3basedir
        self.songdb = songdb
        self.keybindings = keybindings
        self.searchstring = None
        self.lastadded = None

        window.window.__init__(self,
                               screen, h, w, y, x, "MP3s", hasscrollbar)
        
        self.items = filelist.dbbasedfilelist(self, mp3basedir, songdb)

        events.ec.subscribe(events.keypressed, self.keypressed)
        events.ec.subscribe(events.activewinchanged, self.activewinchanged)

    # event handler

    def keypressed(self, event):
        if self.active:
            key = event.key
            if key in self.keybindings["selectnext"]:
                self.items.selectnext()
            elif key in self.keybindings["selectprev"]:
                self.items.selectprev()
            elif key in self.keybindings["selectnextpage"]:
                self.items.selectnextpage()
            elif key in self.keybindings["selectprevpage"]:
                self.items.selectprevpage()
            elif key in self.keybindings["selectfirst"]:
                self.items.selectfirst()
            elif key in self.keybindings["selectlast"]:
                self.items.selectlast()
            elif self.items.selected is not None and \
                 self.items.isdirselected() and \
                 key in self.keybindings["dirdown"]:
                self.items.dirdown()
            elif key in self.keybindings["dirup"]:
                self.items.dirup()
            elif self.items.selected is not None and \
                 key in self.keybindings["addsongtoplaylist"]:
                songtoadd = self.items.getselected()
                if songtoadd is not self.lastadded:
                    self.lastadded = songtoadd
                    events.ec.notify(events.playlistaddsong(songtoadd))
                    self.items.selectrelative(+1)
            elif key in self.keybindings["adddirtoplaylist"]:
                if self.items.selected is not None:
                    itemtoadd = self.items.getselected()
                    if itemtoadd is not self.lastadded:
                        self.lastadded = itemtoadd
                        
                        if self.items.isdirselected():
                            self.items.insertrecursiveselection()
                        else:
                            events.ec.notify(events.playlistaddsong(itemtoadd))
                            
                        self.items.selectrelative(+1)
            elif key in self.keybindings["activateplaylist"]:
                events.ec.notify(events.activateplaylist())
            elif key in self.keybindings["toggleview"]:
                if isinstance(self.items, filelist.dirbasedfilelist):
                    del self.items
                    self.items = filelist.dbbasedfilelist(self, self.mp3base, self.songdb)
                else:
                    del self.items
                    self.items = filelist.dirbasedfilelist(self, self.mp3base, self.songdb)
            elif key in self.keybindings["generaterandomlist"]:
                self.items.generaterandomlist()
            elif key in self.keybindings["search"]:
                events.ec.notify(events.requestinput("Search: ", self.searchhandler))
            elif ord("a")<=key-1024<=ord("z") or ord("A")<=key-1024<=ord("Z") :
                self.items.selectbyletter(chr(key-1024))
            else:
                return
        
            self.update()
            raise events.TerminateEventProcessing


    def activewinchanged(self, event):
        self.items.active = self.active = event.active is self
        if self.active:
            self.lastadded = None
            events.ec.notify(events.selectionchanged(self.items.getselected()))
        self.update()
        
    def updatestatusbar(self):
        if self.items.getselected() is None:
            statusbar = []
        elif self.items.isdirselected():
            if len(self.items.shistory)==0:
                statusbar = [ ("Right", curses.A_BOLD),
                              (": enter dir | ", curses.A_NORMAL) ]
            else:
                statusbar = [ ("Left/Right", curses.A_BOLD),
                              (": exit/enter dir | ", curses.A_NORMAL) ]
            statusbar += [
                           ("i", curses.A_BOLD),
                           (": add dir | ", curses.A_NORMAL),
                         ]
        else:
            statusbar = [ ("Left", curses.A_BOLD),
                          (": exit dir | ", curses.A_NORMAL),
                          ("Enter/Right/i", curses.A_BOLD),
                          (":a add song | ", curses.A_NORMAL),
                        ]

        statusbar += [ ("TAB", curses.A_BOLD),
                       (": switch to playlist | ", curses.A_NORMAL),
                       ("p", curses.A_BOLD),
                       (": play", curses.A_NORMAL),
                     ]

        events.ec.notify(events.updatestatusbar(statusbar))

    def updatescrollbar(self):
        self.drawscrollbar(self.items.top, len(self.items))

    def searchhandler(self, searchstring, key):
        if key==ord("\n"):
            self.items.selectbysearchstring(searchstring)
            self.update()

    # window update method

    def update(self):
        self.settitle(self.items.selectionpath())
        window.window.update(self)

        if self.active:
            self.updatestatusbar()
        
        for i in range(self.items.top, self.items.top+self.ih):
            attr = curses.A_NORMAL
            if i<len(self.items):
                item = self.items[i]
                if item.isdir():
                    name = item.name + "/"
                    attr = curses.A_BOLD
                else:
                    name = item.title
                if i==self.items.selected and self.active:
                    attr |= curses.A_REVERSE
            else:
                name = ""
            self.win.addstr(i-self.items.top+1, 1, name.ljust(self.iw)[:self.iw], attr)

        self.updatescrollbar()
            
        self.refresh()
