#!/usr/bin/env python

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os
import time
import xmms

import events

# player states

STOP     = 0
WANTPLAY = 1
PAUSE    = 3 # not used yet
PLAY     = 4

class player:
    def __init__(self, session=0, autoplay=0, noqueue=0):
        self.session = session
        self.autoplay = autoplay
        if autoplay:
            self.state = WANTPLAY
        else:
            self.state = STOP
        self.noqueue = noqueue
        self.initxmms()
        events.ec.subscribe(events.timer, self.timer)
        events.ec.subscribe(events.playerstart, self.playerstart)
        events.ec.subscribe(events.playerstop, self.playerstop)
        events.ec.subscribe(events.playerforward, self.playerforward)
        events.ec.subscribe(events.nextsong, self.nextsong)

    def initxmms(self):
        if not xmms.is_running(self.session):
            abs_prog_name = xmms._find_and_check_executable("xmms")
            if not abs_prog_name:
                raise xmms.ExecutableNotFound("can't find XMMS executable")
            os.system(abs_prog_name + " >/dev/null 2>/dev/null &")
            while not xmms.is_running(self.session):
                time.sleep(0.2)

        xmms.playlist_clear(self.session)
        xmms.main_win_toggle(0, self.session)
        xmms.pl_win_toggle(0, self.session)
        xmms.eq_win_toggle(0, self.session)

    # event handler

    def timer(self, event):
        if xmms.is_playing(self.session):
            pos = xmms.get_playlist_pos(self.session)
            title = xmms.get_playlist_title(pos, self.session)
            ttime = xmms.get_playlist_time(pos, self.session)
            ptime = xmms.get_output_time(self.session)

            # fill up xmms playlist if necessary
            if ttime-ptime<20*1000 and xmms.get_playlist_length()<2:
                events.ec.notify(events.requestnextsong(self))
            if pos>0:
                xmms.playlist_delete(0, self.session)

            events.ec.notify(events.playbackinfochanged(self, 1, title, ptime, ttime))
        else:
            events.ec.notify(events.playbackinfochanged(self))
            if self.state==WANTPLAY:
                events.ec.notify(events.requestnextsong(self))

    def nextsong(self, event):
        if event.player is self:
            if event.song:
                if self.noqueue:
                    xmms.playlist_clear(self.session)
                xmms.playlist_add((event.song.path,), self.session)
                if not xmms.is_playing():
                    xmms.play(self.session)
            else:
                if self.autoplay:
                    self.state = WANTPLAY
                else:
                    self.state = STOP
                    # xmms.playlist_clear(self.session)

    def playerstart(self, event):
        if event.player is self:
            if not xmms.is_playing(self.session):
                xmms.playlist_clear(self.session)
                events.ec.notify(events.requestnextsong(self))
                if xmms.get_playlist_length(self.session)>0:
                    xmms.play(self.session)

    def playerstop(self, event):
        if event.player is self:
            self.state = STOP
            xmms.playlist_clear(self.session)

    def playerforward(self, event):
        if event.player is self:
            if xmms.is_playing(self.session):
                events.ec.notify(events.requestnextsong(self))
                # wait a little for xmms
                time.sleep(0.1)
            xmms.playlist_next(self.session)

    def playerquit(self):
        xmms.quit(self.session)

