#!/usr/bin/env python

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import curses

import events
import playlist
import window

from helper import formattime, debug

class playlistwin(window.window):
    def __init__(self,
                 screen, h, w, y, x, hasscrollbar,
                 songdb, logfilename, keybindings):
        self.songdb = songdb
        
        # we both need the playlist and the global keybindings
        self.keybindings = keybindings
        
        window.window.__init__(self,
                               screen, h, w, y, x, "Playlist", hasscrollbar)
        
        self.playlist = playlist.playlist(self, songdb, logfilename)
        
        events.ec.subscribe(events.keypressed, self.keypressed)
        events.ec.subscribe(events.activewinchanged, self.activewinchanged)


    # event handlers

    def keypressed(self, event):
        if self.active:
            key = event.key

            if key in self.keybindings["selectnext"]:
                self.playlist.selectnext()
            elif key in self.keybindings["selectprev"]:
                self.playlist.selectprev()
            elif key in self.keybindings["selectnextpage"]:
                self.playlist.selectnextpage()
            elif key in self.keybindings["selectprevpage"]:
                self.playlist.selectprevpage()
            elif key in self.keybindings["selectfirst"]:
                self.playlist.selectfirst()
            elif key in self.keybindings["selectlast"]:
                self.playlist.selectlast()
            elif key in self.keybindings["activatefilelist"]:
                events.ec.notify(events.activatefilelist())
            elif key in self.keybindings["moveitemup"]:
                self.playlist.moveitemup()
            elif key in self.keybindings["moveitemdown"]:
                self.playlist.moveitemdown()
            elif key in self.keybindings["deleteitem"]:
                self.playlist.deleteselected()
            else:
                return
            
            self.update()
            raise events.TerminateEventProcessing

    def activewinchanged(self, event):
        self.playlist.active = self.active = event.active is self
        if self.active:
            events.ec.notify(events.selectionchanged(self.playlist.getselected()))
        self.update()

    def updatestatusbar(self):
        if self.playlist.selected is not None:
            statusbar = [ ("d", curses.A_BOLD),
                          (": delete item | ", curses.A_NORMAL),
                          ("+", curses.A_BOLD),
                          ("/", curses.A_NORMAL),
                          ("-", curses.A_BOLD),
                          (": move item up/down | ", curses.A_NORMAL),
                          ]
        else:
            statusbar = []

        statusbar += [ ("TAB", curses.A_BOLD),
                       (": switch to database | ", curses.A_NORMAL),
                       ("p", curses.A_BOLD),
                       (": play", curses.A_NORMAL),
                       ]

        events.ec.notify(events.updatestatusbar(statusbar))

    def updatescrollbar(self):
        self.drawscrollbar(self.playlist.top, len(self.playlist))

    # window update method

    def update(self):
        self.settitle("Playlist (-%s/%s)" % (formattime(self.playlist.ttime-
                                                        self.playlist.ptime),
                                             formattime(self.playlist.ttime)))

        window.window.update(self)
        if self.active:
            self.updatestatusbar()
            
        for i in range(self.playlist.top, self.playlist.top+self.ih):
            attr = curses.A_NORMAL

            if i<len(self.playlist):
                song = self.playlist[i]
                name = song.title
                if not song.hasbeenplayed():
                    attr = curses.A_BOLD
                if i==self.playlist.selected and self.active:
                    attr |= curses.A_REVERSE
            else:
                name = ""
            self.win.addstr(i-self.playlist.top+1, 1,
                            name.ljust(self.iw)[:self.iw],
                            attr)


        self.updatescrollbar()

        self.refresh()
