#!/usr/bin/env python

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import curses

class window:
    def __init__(self, screen, h, w, y, x, title=None, hasscrollbar=0):
        self.screen = screen
        self.win = screen.subwin(h, w, y, x)
        self.h = h
        self.w = w
        self.x = x
        self.y = y
        self.hasscrollbar = hasscrollbar

        # width and height of inner part of window
        self.ih = h-2
        self.iw = w-2
        if self.hasscrollbar:
            self.iw -= 1
            
        self.active = 0
        
        self.settitle(title)

    def settitle(self, title):
        if title is not None:
            self.title = title[:self.w-8]
        else:
            self.title = None

    def drawscrollbar(self, top, total):
        if self.hasscrollbar:
            height = self.ih
            xpos = self.iw+1

            self.win.addch(1, xpos,
                           top!=0 and curses.ACS_UARROW or " ",
                           curses.A_BOLD)
            self.win.addch(height, xpos,
                           top+height<total and curses.ACS_DARROW or " ",
                           curses.A_BOLD)
        
            if total>height:
                scrollbarchar = "#"
                scrollbarbegin = 2 + height*top/total
                scrollbarend = scrollbarbegin + height*height/total
            else:
                scrollbarchar = " "
                scrollbarbegin = scrollbarend = 0
            
            for line in range(2, height):
                self.win.addch(line, xpos,
                               scrollbarchar,
                               line>=scrollbarbegin and line<=scrollbarend
                               and curses.A_BOLD
                               or curses.A_NORMAL)
    

    def update(self):
        if self.title:
            if self.active:
                topborder = ord("=")
            else:
                topborder = 0
            self.win.border(0, 0, topborder )
            t = self.title[:self.w-4]
            pos = (self.w-4-len(t))/2 
            self.win.addstr(0, pos, "[ %s ]" % t, curses.A_BOLD)

    def refresh(self):
        self.win.noutrefresh()

